@@ -1,5 +1,5 @@
-/* estream.c - Extended stream I/O/ Library
- * Copyright (C) 2004 g10 Code GmbH
+/* estream.c - Extended Stream I/O Library
+ * Copyright (C) 2004, 2006 g10 Code GmbH
  *
  * This file is part of Libestream.
  *
@@ -30,6 +30,7 @@
 #include <sys/types.h>
 #include <sys/file.h>
 #include <sys/stat.h>
+#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
@@ -39,6 +40,11 @@
 #include <stddef.h>
 #include <assert.h>
 
+#ifdef WITHOUT_GNU_PTH /* Give the Makefile a chance to build without Pth.  */
+#undef HAVE_PTH
+#undef USE_GNU_PTH
+#endif
+
 #ifdef HAVE_PTH
 # include <pth.h>
 #endif
@@ -149,8 +155,13 @@ struct estream_internal
     unsigned int eof: 1;
   } indicators;
   unsigned int deallocate_buffer: 1;
+  unsigned int print_err: 1;     /* Error in print_fun_writer.  */
+  int print_errno;               /* Errno from print_fun_writer.  */
+  size_t print_ntotal;           /* Bytes written from in print_fun_writer. */
+  FILE *print_fp;                /* Stdio stream used by print_fun_writer.  */
 };
 
+
 typedef struct estream_internal *estream_internal_t;
 
 #define ESTREAM_LOCK(stream) ESTREAM_MUTEX_LOCK (stream->intern->lock)
@@ -559,7 +570,7 @@ static es_cookie_io_functions_t estream_functions_mem =
     es_func_mem_read,
     es_func_mem_write,
     es_func_mem_seek,
-    es_func_mem_destroy,
+    es_func_mem_destroy
   };
 
 /* Implementation of fd I/O.  */
@@ -916,6 +927,10 @@ es_initialize (estream_t stream,
   stream->intern->func_close = functions.func_close;
   stream->intern->strategy = _IOFBF;
   stream->intern->fd = fd;
+  stream->intern->print_err = 0;
+  stream->intern->print_errno = 0;
+  stream->intern->print_ntotal = 0;
+  stream->intern->print_fp = NULL;
   stream->intern->indicators.err = 0;
   stream->intern->indicators.eof = 0;
   stream->intern->deallocate_buffer = 0;
@@ -934,6 +949,14 @@ es_deinitialize (estream_t stream)
   es_cookie_close_function_t func_close;
   int err, tmp_err;
 
+  if (stream->intern->print_fp)
+    {
+      int save_errno = errno;
+      fclose (stream->intern->print_fp);
+      stream->intern->print_fp = NULL;
+      errno = save_errno;
+    }
+
   func_close = stream->intern->func_close;
 
   err = 0;
@@ -941,6 +964,7 @@ es_deinitialize (estream_t stream)
     SET_UNLESS_NONZERO (err, tmp_err, es_flush (stream));
   if (func_close)
     SET_UNLESS_NONZERO (err, tmp_err, (*func_close) (stream->intern->cookie));
+
   
   return err;
 }
@@ -1402,16 +1426,19 @@ es_writen (estream_t ES__RESTRICT stream,
   if (! (stream->flags & ES_FLAG_WRITING))
     {
       /* Switching to writing mode -> discard input data and seek to
-	 position at which reading has stopped.  */
-
-      err = es_seek (stream, 0, SEEK_CUR, NULL);
-      if (err)
-	{
-	  if (errno == ESPIPE)
-	    err = 0;
-	  else
-	    goto out;
-	}
+	 position at which reading has stopped.  We can do this only
+	 if a seek function has been registered. */
+      if (stream->intern->func_seek)
+        {
+          err = es_seek (stream, 0, SEEK_CUR, NULL);
+          if (err)
+            {
+              if (errno == ESPIPE)
+                err = 0;
+              else
+                goto out;
+            }
+        }
     }
 
   switch (stream->intern->strategy)
@@ -1498,9 +1525,9 @@ es_skip (estream_t stream, size_t size)
 
 
 static int
-es_read_line (estream_t ES__RESTRICT stream, size_t max_length,
-	      char *ES__RESTRICT *ES__RESTRICT line,
-	      size_t *ES__RESTRICT line_length)
+doreadline (estream_t ES__RESTRICT stream, size_t max_length,
+             char *ES__RESTRICT *ES__RESTRICT line,
+             size_t *ES__RESTRICT line_length)
 {
   size_t space_left;
   size_t line_size;
@@ -1622,13 +1649,80 @@ es_read_line (estream_t ES__RESTRICT stream, size_t max_length,
 }
 
 
+/* Helper for esprint. */
+#if defined(HAVE_FOPENCOOKIE) || defined(HAVE_FUNOPEN)
+static int 
+print_fun_writer (void *cookie_arg, const char *buffer, size_t size)
+{
+  estream_t stream = cookie_arg;
+  size_t nwritten;
+  
+  /* We don't return an error but let es_print check whether an error
+     has occured.  Internally we skip everything after an error. */
+  if (!stream->intern->print_err)
+    {
+      if (es_writen (stream, buffer, size, &nwritten))
+        {
+          stream->intern->print_err = 1;
+          stream->intern->print_errno = errno;
+        }
+      else
+        stream->intern->print_ntotal += nwritten;
+    }
+  return 0;
+}
+#endif /* HAVE_FOPENCOOKIE || HAVE_FUNOPEN */
+
+
+/* The core of our printf function.  This is called in locked state. */
 static int
 es_print (estream_t ES__RESTRICT stream,
 	  const char *ES__RESTRICT format, va_list ap)
 {
+#if defined(HAVE_FOPENCOOKIE) || defined(HAVE_FUNOPEN)
+
+  if (!stream->intern->print_fp)
+    {
+#ifdef HAVE_FOPENCOOKIE
+      {
+        cookie_io_functions_t io = { NULL };
+        io.write = print_fun_writer;
+        
+        stream->intern->print_fp = fopencookie (stream, "w", io);
+      }
+#else /*!HAVE_FOPENCOOKIE*/
+      stream->intern->print_fp = funopen (stream, NULL,
+                                          print_fun_writer, NULL, NULL);
+#endif /*!HAVE_FOPENCOOKIE*/
+      if (!stream->intern->print_fp)
+        return -1;
+    }
+
+  stream->intern->print_err = 0;
+  stream->intern->print_errno = 0;
+  stream->intern->print_ntotal = 0;
+
+  if ( vfprintf (stream->intern->print_fp, format, ap) < 0 
+       || fflush (stream->intern->print_fp) )
+    {
+      stream->intern->print_errno = errno;
+      stream->intern->print_err = 1;
+      fclose (stream->intern->print_fp);
+      stream->intern->print_fp = NULL;
+    }
+  if (stream->intern->print_err)
+    {
+      errno = stream->intern->print_errno;
+      return -1;
+    }
+
+  return (int)stream->intern->print_ntotal;
+  
+#else /* No funopen or fopencookie. */
+  
   char data[BUFFER_BLOCK_SIZE];
-  size_t bytes_written;
   size_t bytes_read;
+  size_t bytes_written;
   FILE *tmp_stream;
   int err;
 
@@ -1672,11 +1766,11 @@ es_print (estream_t ES__RESTRICT stream,
     goto out;
 
  out:
-
   if (tmp_stream)
     fclose (tmp_stream);
 
   return err ? -1 : bytes_written;
+#endif /* no funopen or fopencookie */
 }
 
 
@@ -2383,7 +2477,7 @@ es_fgets (char *ES__RESTRICT s, int n, estream_t ES__RESTRICT stream)
       int err;
       
       ESTREAM_LOCK (stream);
-      err = es_read_line (stream, n, &s, NULL);
+      err = doreadline (stream, n, &s, NULL);
       ESTREAM_UNLOCK (stream);
       if (! err)
 	ret = s;
@@ -2417,7 +2511,7 @@ es_getline (char *ES__RESTRICT *ES__RESTRICT lineptr, size_t *ES__RESTRICT n,
   int err;
 
   ESTREAM_LOCK (stream);
-  err = es_read_line (stream, 0, &line, &line_n);
+  err = doreadline (stream, 0, &line, &line_n);
   ESTREAM_UNLOCK (stream);
   if (err)
     goto out;
@@ -2463,6 +2557,129 @@ es_getline (char *ES__RESTRICT *ES__RESTRICT lineptr, size_t *ES__RESTRICT n,
 }
 
 
+
+/* Same as fgets() but if the provided buffer is too short a larger
+   one will be allocated.  This is similar to getline. A line is
+   considered a byte stream ending in a LF.
+
+   If MAX_LENGTH is not NULL, it shall point to a value with the
+   maximum allowed allocation.  
+
+   Returns the length of the line. EOF is indicated by a line of
+   length zero. A truncated line is indicated my setting the value at
+   MAX_LENGTH to 0.  If the returned value is less then 0 not enough
+   memory was enable or another error occurred; ERRNO is then set
+   accordingly.
+
+   If a line has been truncated, the file pointer is moved forward to
+   the end of the line so that the next read starts with the next
+   line.  Note that MAX_LENGTH must be re-initialzied in this case.
+
+   The caller initially needs to provide the address of a variable,
+   initialized to NULL, at ADDR_OF_BUFFER and don't change this value
+   anymore with the following invocations.  LENGTH_OF_BUFFER should be
+   the address of a variable, initialized to 0, which is also
+   maintained by this function.  Thus, both paramaters should be
+   considered the state of this function.
+
+   Note: The returned buffer is allocated with enough extra space to
+   allow the caller to append a CR,LF,Nul.  The buffer should be
+   released using es_free.
+ */
+ssize_t
+es_read_line (estream_t stream, 
+              char **addr_of_buffer, size_t *length_of_buffer,
+              size_t *max_length)
+{
+  int c;
+  char  *buffer = *addr_of_buffer;
+  size_t length = *length_of_buffer;
+  size_t nbytes = 0;
+  size_t maxlen = max_length? *max_length : 0;
+  char *p;
+
+  if (!buffer)
+    { 
+      /* No buffer given - allocate a new one. */
+      length = 256;
+      buffer = MEM_ALLOC (length);
+      *addr_of_buffer = buffer;
+      if (!buffer)
+        {
+          *length_of_buffer = 0;
+          if (max_length)
+            *max_length = 0;
+          return -1;
+        }
+      *length_of_buffer = length;
+    }
+
+  if (length < 4)
+    {
+      /* This should never happen. If it does, the fucntion has been
+         called with wrong arguments. */
+      errno = EINVAL;
+      return -1;
+    }
+  length -= 3; /* Reserve 3 bytes for CR,LF,EOL. */
+
+  ESTREAM_LOCK (stream);
+  p = buffer;
+  while  ((c = es_getc_unlocked (stream)) != EOF)
+    {
+      if (nbytes == length)
+        { 
+          /* Enlarge the buffer. */
+          if (maxlen && length > maxlen) 
+            {
+              /* We are beyond our limit: Skip the rest of the line. */
+              while (c != '\n' && (c=es_getc_unlocked (stream)) != EOF)
+                ;
+              *p++ = '\n'; /* Always append a LF (we reserved some space). */
+              nbytes++;
+              if (max_length)
+                *max_length = 0; /* Indicate truncation. */
+              break; /* the while loop. */
+            }
+          length += 3; /* Adjust for the reserved bytes. */
+          length += length < 1024? 256 : 1024;
+          *addr_of_buffer = MEM_REALLOC (buffer, length);
+          if (!*addr_of_buffer)
+            {
+              int save_errno = errno;
+              MEM_FREE (buffer); 
+              *length_of_buffer = *max_length = 0;
+              ESTREAM_UNLOCK (stream);
+              errno = save_errno;
+              return -1;
+            }
+          buffer = *addr_of_buffer;
+          *length_of_buffer = length;
+          length -= 3; 
+          p = buffer + nbytes;
+	}
+      *p++ = c;
+      nbytes++;
+      if (c == '\n')
+        break;
+    }
+  *p = 0; /* Make sure the line is a string. */
+  ESTREAM_UNLOCK (stream);
+
+  return nbytes;
+}
+
+/* Wrapper around free() to match the memory allocation system used
+   by estream.  Should be used for all buffers returned to the caller
+   by libestream. */
+void
+es_free (void *a)
+{
+  if (a)
+    MEM_FREE (a);
+}
+
+
 int
 es_vfprintf (estream_t ES__RESTRICT stream, const char *ES__RESTRICT format,
 	     va_list ap)
@@ -2613,3 +2830,4 @@ es_opaque_get (estream_t stream)
 
   return opaque;
 }
+
