@@ -1,5 +1,5 @@
 /* miscellaneous.c - Stuff not fitting elsewhere
- *	Copyright (C) 2003 Free Software Foundation, Inc.
+ *	Copyright (C) 2003, 2006 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -15,7 +15,8 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
+ * USA.
  */
 
 #include <config.h>
@@ -47,6 +48,7 @@ print_fname_stdin (const char *s)
     return s;
 }
 
+/* fixme: Globally replace it by print_sanitized_buffer. */
 void
 print_string( FILE *fp, const byte *p, size_t n, int delim )
 {
@@ -81,6 +83,7 @@ is_file_compressed (const char *s, int *ret_rc)
     iobuf_t a;
     byte buf[4];
     int i, rc = 0;
+    int overflow;
 
     struct magic_compress_s {
         size_t len;
@@ -91,16 +94,16 @@ is_file_compressed (const char *s, int *ret_rc)
         { 4, { 0x50, 0x4b, 0x03, 0x04 } }, /* (pk)zip */
     };
     
-    if ( !s || (*s == '-' && !s[1]) || !ret_rc )
+    if ( iobuf_is_pipe_filename (s) || !ret_rc )
         return 0; /* We can't check stdin or no file was given */
 
     a = iobuf_open( s );
     if ( a == NULL ) {
-        *ret_rc = gpg_error_from_errno (errno);
+        *ret_rc = gpg_error_from_syserror ();
         return 0;
     }
 
-    if ( iobuf_get_filelength( a ) < 4 ) {
+    if ( iobuf_get_filelength( a, &overflow ) < 4 && !overflow) {
         *ret_rc = 0;
         goto leave;
     }
@@ -124,4 +127,25 @@ leave:
 }
 
 
+/* Try match against each substring of multistr, delimited by | */
+int
+match_multistr (const char *multistr,const char *match)
+{
+  do
+    {
+      size_t seglen = strcspn (multistr,"|");
+      if (!seglen)
+	break;
+      /* Using the localized strncasecmp! */
+      if (strncasecmp(multistr,match,seglen)==0)
+	return 1;
+      multistr += seglen;
+      if (*multistr == '|')
+	multistr++;
+    }
+  while (*multistr);
+
+  return 0;
+}
+
 
