@@ -64,6 +64,7 @@
 #include "main.h"
 #include "photoid.h"
 #include "options.h"
+#include "call-agent.h"
 #include "i18n.h"
 
 
@@ -297,7 +298,6 @@ checksum_mpi (gcry_mpi_t a)
   u16 csum;
   byte *buffer;
   unsigned int nbytes;
-  unsigned int nbits;
 
   if ( gcry_mpi_print (GCRYMPI_FMT_PGP, NULL, 0, &nbytes, a) )
     BUG ();
@@ -308,9 +308,7 @@ checksum_mpi (gcry_mpi_t a)
             gcry_xmalloc_secure (nbytes) : gcry_xmalloc (nbytes));
   if ( gcry_mpi_print (GCRYMPI_FMT_PGP, buffer, nbytes, NULL, a) )
     BUG ();
-  nbits = gcry_mpi_get_nbits (a);
-  csum = checksum_u16 (nbits);
-  csum += checksum (buffer, nbytes);
+  csum = checksum (buffer, nbytes);
   xfree (buffer);
   return csum;
 }
@@ -493,7 +491,9 @@ idea_cipher_warn(int show)
 }
 #endif
 
-static unsigned long get_signature_count(PKT_secret_key *sk)
+
+static unsigned long 
+get_signature_count (PKT_secret_key *sk)
 {
 #ifdef ENABLE_CARD_SUPPORT
   if(sk && sk->is_protected && sk->protect.s2k.mode==1002)
