@@ -114,14 +114,16 @@ struct
   int pgp2_workarounds;
   int shm_coprocess;
   const char *set_filename;
-  STRLIST comments;
+  strlist_t comments;
   int throw_keyid;
   const char *photo_viewer;
   int s2k_mode;
   int s2k_digest_algo;
   int s2k_cipher_algo;
-  int simple_sk_checksum; /* create the deprecated rfc2440 secret
-			     key protection*/
+  unsigned char s2k_count; /* This is the encoded form, not the raw
+			      count */
+  int simple_sk_checksum; /* create the deprecated rfc2440 secret key
+			     protection */
   int not_dash_escaped;
   int escape_from;
   int lock_once;
@@ -134,7 +136,7 @@ struct
     char *port;
     char *path;
     char *opaque;
-    STRLIST options;
+    strlist_t options;
     struct
     {
       unsigned int direct_uri:1;
@@ -146,7 +148,7 @@ struct
     unsigned int options;
     unsigned int import_options;
     unsigned int export_options;
-    STRLIST other;
+    strlist_t other;
   } keyserver_options;
   int exec_disable;
   int exec_path_set;
@@ -154,7 +156,8 @@ struct
   unsigned int export_options;
   unsigned int list_options;
   unsigned int verify_options;
-  char *def_preference_list;
+  const char *def_preference_list;
+  const char *def_keyserver_url;
   prefitem_t *personal_cipher_prefs;
   prefitem_t *personal_digest_prefs;
   prefitem_t *personal_compress_prefs;
@@ -165,11 +168,11 @@ struct
   int interactive;
   struct notation *sig_notations;
   struct notation *cert_notations;
-  STRLIST sig_policy_url;
-  STRLIST cert_policy_url;
-  STRLIST sig_keyserver_url;
-  STRLIST cert_subpackets;
-  STRLIST sig_subpackets;
+  strlist_t sig_policy_url;
+  strlist_t cert_policy_url;
+  strlist_t sig_keyserver_url;
+  strlist_t cert_subpackets;
+  strlist_t sig_subpackets;
   int allow_non_selfsigned_uid;
   int allow_freeform_uid;
   int no_literal;
@@ -183,7 +186,7 @@ struct
   int command_fd;
   const char *override_session_key;
   int show_session_key;
-  int use_agent;
+
   const char *gpg_agent_info;
   int try_all_secrets;
   int no_expensive_trust_checks;
