@@ -27,10 +27,6 @@
 #endif
 #define GPG_ERR_SOURCE_DEFAULT  GPG_ERR_SOURCE_SCD
 #include <gpg-error.h>
-#define map_assuan_err(a) \
-        map_assuan_err_with_source (GPG_ERR_SOURCE_DEFAULT, (a))
-
-#include <errno.h>
 
 #include <time.h>
 #include <gcrypt.h>
@@ -38,10 +34,21 @@
 #include "../common/errors.h"
 
 
-#define MAX_DIGEST_LEN 24 
+/* To convey some special hash algorithms we use algorithm numbers
+   reserved for application use. */
+#ifndef GCRY_MD_USER
+#define GCRY_MD_USER 1024
+#endif
+#define GCRY_MD_USER_TLS_MD5SHA1 (GCRY_MD_USER+1)
+
+/* Maximum length of a digest.  */
+#define MAX_DIGEST_LEN 36
+
+
 
 /* A large struct name "opt" to keep global flags. */
-struct {
+struct
+{
   unsigned int debug; /* Debug flags (DBG_foo_VALUE). */
   int verbose;        /* Verbosity level. */
   int quiet;          /* Be as quiet as possible. */
@@ -103,9 +110,7 @@ struct server_control_s
   } in_data;  
 };
 
-typedef struct server_control_s *CTRL;
 typedef struct server_control_s *ctrl_t;
-typedef struct app_ctx_s *APP;
 typedef struct app_ctx_s *app_t;
 
 /*-- scdaemon.c --*/
@@ -115,7 +120,8 @@ const char *scd_get_socket_name (void);
 
 /*-- command.c --*/
 void scd_command_handler (int);
-void send_status_info (CTRL ctrl, const char *keyword, ...);
+void send_status_info (ctrl_t ctrl, const char *keyword, ...)
+     GNUPG_GCC_A_SENTINEL(1);
 void scd_update_reader_status_file (void);
 
 
