@@ -27,20 +27,17 @@
 #endif
 #define GPG_ERR_SOURCE_DEFAULT  GPG_ERR_SOURCE_GPGSM
 #include <gpg-error.h>
-#define map_assuan_err(a) \
-        map_assuan_err_with_source (GPG_ERR_SOURCE_DEFAULT, (a))
 
 
 #include <ksba.h>
 #include "../common/util.h"
 #include "../common/errors.h"
 
-#define OUT_OF_CORE(a) (gpg_error (gpg_err_code_from_errno ((a))))
-
 #define MAX_DIGEST_LEN 24 
 
-/* A large struct named "opt" to keep global flags */
-struct {
+/* A large struct named "opt" to keep global flags. */
+struct 
+{
   unsigned int debug; /* debug flags (DBG_foo_VALUE) */
   int verbose;      /* verbosity level */
   int quiet;        /* be as quiet as possible */
@@ -119,6 +116,7 @@ struct {
 } opt;
 
 
+/* Debug values and macros.  */
 #define DBG_X509_VALUE    1	/* debug x.509 data reading/writing */
 #define DBG_MPI_VALUE	  2	/* debug mpi details */
 #define DBG_CRYPTO_VALUE  4	/* debug low level crypto */
@@ -135,11 +133,14 @@ struct {
 #define DBG_HASHING (opt.debug & DBG_HASHING_VALUE)
 #define DBG_ASSUAN   (opt.debug & DBG_ASSUAN_VALUE)
 
+/* Forward declaration for an object defined in server.c */
 struct server_local_s;
 
-/* Note that the default values for this are set by
-   gpgsm_init_default_ctrl() */
-struct server_control_s {
+/* Session control object.  This object is passed down to most
+   functions.  Note that the default values for it are set by
+   gpgsm_init_default_ctrl(). */
+struct server_control_s
+{
   int no_server;      /* We are not running under server control */
   int  status_fd;     /* Only for non-server mode */
   struct server_local_s *server_local;
@@ -161,22 +162,33 @@ struct server_control_s {
                          signer) */
   int use_ocsp;       /* Set to true if OCSP should be used. */
 };
-typedef struct server_control_s *CTRL;
 typedef struct server_control_s *ctrl_t;
 
-/* data structure used in base64.c */
+
+/* Data structure used in base64.c. */
 typedef struct base64_context_s *Base64Context;
 
 
-struct certlist_s {
+/* An object to keep a list of certificates. */
+struct certlist_s 
+{
   struct certlist_s *next;
   ksba_cert_t cert;
   int is_encrypt_to; /* True if the certificate has been set through
                         the --encrypto-to option. */
 };
-typedef struct certlist_s *CERTLIST;
 typedef struct certlist_s *certlist_t;
 
+
+/* A structure carrying information about trusted root certificates. */
+struct rootca_flags_s
+{
+  unsigned int relax:1;  /* Relax checking of root certificates.  */
+
+};
+
+
+
 /*-- gpgsm.c --*/
 void gpgsm_exit (int rc);
 void gpgsm_init_default_ctrl (struct server_control_s *ctrl);
@@ -218,6 +230,8 @@ void gpgsm_print_time (FILE *fp, ksba_isotime_t t);
 void gpgsm_print_name2 (FILE *fp, const char *string, int translate);
 void gpgsm_print_name (FILE *fp, const char *string);
 
+void gpgsm_cert_log_name (const char *text, ksba_cert_t cert);
+
 void gpgsm_dump_cert (const char *text, ksba_cert_t cert);
 void gpgsm_dump_serial (ksba_const_sexp_t p);
 void gpgsm_dump_time (ksba_isotime_t t);
@@ -264,7 +278,7 @@ void gpgsm_release_certlist (certlist_t list);
 int gpgsm_find_cert (const char *name, ksba_sexp_t keyid, ksba_cert_t *r_cert);
 
 /*-- keylist.c --*/
-gpg_error_t gpgsm_list_keys (ctrl_t ctrl, STRLIST names,
+gpg_error_t gpgsm_list_keys (ctrl_t ctrl, strlist_t names,
                              FILE *fp, unsigned int mode);
 
 /*-- import.c --*/
@@ -273,22 +287,22 @@ int gpgsm_import_files (ctrl_t ctrl, int nfiles, char **files,
                         int (*of)(const char *fname));
 
 /*-- export.c --*/
-void gpgsm_export (ctrl_t ctrl, STRLIST names, FILE *fp);
+void gpgsm_export (ctrl_t ctrl, strlist_t names, FILE *fp);
 void gpgsm_p12_export (ctrl_t ctrl, const char *name, FILE *fp);
 
 /*-- delete.c --*/
-int gpgsm_delete (ctrl_t ctrl, STRLIST names);
+int gpgsm_delete (ctrl_t ctrl, strlist_t names);
 
 /*-- verify.c --*/
 int gpgsm_verify (ctrl_t ctrl, int in_fd, int data_fd, FILE *out_fp);
 
 /*-- sign.c --*/
 int gpgsm_get_default_cert (ctrl_t ctrl, ksba_cert_t *r_cert);
-int gpgsm_sign (ctrl_t ctrl, CERTLIST signerlist,
+int gpgsm_sign (ctrl_t ctrl, certlist_t signerlist,
                 int data_fd, int detached, FILE *out_fp);
 
 /*-- encrypt.c --*/
-int gpgsm_encrypt (ctrl_t ctrl, CERTLIST recplist, int in_fd, FILE *out_fp);
+int gpgsm_encrypt (ctrl_t ctrl, certlist_t recplist, int in_fd, FILE *out_fp);
 
 /*-- decrypt.c --*/
 int gpgsm_decrypt (ctrl_t ctrl, int in_fd, FILE *out_fp);
@@ -308,14 +322,18 @@ int gpgsm_agent_pksign (ctrl_t ctrl, const char *keygrip, const char *desc,
                         size_t digestlen,
                         int digestalgo,
                         unsigned char **r_buf, size_t *r_buflen);
+int gpgsm_scd_pksign (ctrl_t ctrl, const char *keyid, const char *desc,
+                      unsigned char *digest, size_t digestlen, int digestalgo,
+                      unsigned char **r_buf, size_t *r_buflen);
 int gpgsm_agent_pkdecrypt (ctrl_t ctrl, const char *keygrip, const char *desc,
                            ksba_const_sexp_t ciphertext, 
                            char **r_buf, size_t *r_buflen);
 int gpgsm_agent_genkey (ctrl_t ctrl,
                         ksba_const_sexp_t keyparms, ksba_sexp_t *r_pubkey);
-int gpgsm_agent_readkey (ctrl_t ctrl, const char *hexkeygrip,
+int gpgsm_agent_readkey (ctrl_t ctrl, int fromcard, const char *hexkeygrip,
                          ksba_sexp_t *r_pubkey);
-int gpgsm_agent_istrusted (ctrl_t ctrl, ksba_cert_t cert);
+int gpgsm_agent_istrusted (ctrl_t ctrl, ksba_cert_t cert,
+                           struct rootca_flags_s *rootca_flags);
 int gpgsm_agent_havekey (ctrl_t ctrl, const char *hexkeygrip);
 int gpgsm_agent_marktrusted (ctrl_t ctrl, ksba_cert_t cert);
 int gpgsm_agent_learn (ctrl_t ctrl);
@@ -326,7 +344,7 @@ gpg_error_t gpgsm_agent_get_confirmation (ctrl_t ctrl, const char *desc);
 int gpgsm_dirmngr_isvalid (ctrl_t ctrl,
                            ksba_cert_t cert, ksba_cert_t issuer_cert,
                            int use_ocsp);
-int gpgsm_dirmngr_lookup (ctrl_t ctrl, STRLIST names,
+int gpgsm_dirmngr_lookup (ctrl_t ctrl, strlist_t names,
                           void (*cb)(void*, ksba_cert_t), void *cb_value);
 int gpgsm_dirmngr_run_command (ctrl_t ctrl, const char *command,
                                int argc, char **argv);
