@@ -15,7 +15,8 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
+ * USA.
  */
 
 #include <config.h>
@@ -52,7 +53,7 @@ static gpg_error_t parse_p12 (ctrl_t ctrl, ksba_reader_t reader, FILE **retfp,
 
 
 static void
-print_imported_status (CTRL ctrl, ksba_cert_t cert, int new_cert)
+print_imported_status (ctrl_t ctrl, ksba_cert_t cert, int new_cert)
 {
   char *fpr;
      
@@ -75,7 +76,7 @@ print_imported_status (CTRL ctrl, ksba_cert_t cert, int new_cert)
    4 := "Error storing certificate".
 */
 static void
-print_import_problem (CTRL ctrl, ksba_cert_t cert, int reason)
+print_import_problem (ctrl_t ctrl, ksba_cert_t cert, int reason)
 {
   char *fpr = NULL;
   char buf[25];
@@ -100,7 +101,7 @@ print_import_problem (CTRL ctrl, ksba_cert_t cert, int reason)
 
 
 void
-print_imported_summary (CTRL ctrl, struct stats_s *stats)
+print_imported_summary (ctrl_t ctrl, struct stats_s *stats)
 {
   char buf[14*25];
 
@@ -146,7 +147,7 @@ print_imported_summary (CTRL ctrl, struct stats_s *stats)
 
 
 static void
-check_and_store (CTRL ctrl, struct stats_s *stats, ksba_cert_t cert, int depth)
+check_and_store (ctrl_t ctrl, struct stats_s *stats, ksba_cert_t cert, int depth)
 {
   int rc;
 
@@ -244,7 +245,7 @@ check_and_store (CTRL ctrl, struct stats_s *stats, ksba_cert_t cert, int depth)
 
 
 static int
-import_one (CTRL ctrl, struct stats_s *stats, int in_fd)
+import_one (ctrl_t ctrl, struct stats_s *stats, int in_fd)
 {
   int rc;
   Base64Context b64reader = NULL;
@@ -405,7 +406,7 @@ import_one (CTRL ctrl, struct stats_s *stats, int in_fd)
 
 
 int
-gpgsm_import (CTRL ctrl, int in_fd)
+gpgsm_import (ctrl_t ctrl, int in_fd)
 {
   int rc;
   struct stats_s stats;
@@ -423,7 +424,7 @@ gpgsm_import (CTRL ctrl, int in_fd)
 
 
 int
-gpgsm_import_files (CTRL ctrl, int nfiles, char **files,
+gpgsm_import_files (ctrl_t ctrl, int nfiles, char **files,
                     int (*of)(const char *fname))
 {
   int rc = 0;
@@ -520,7 +521,7 @@ parse_p12 (ctrl_t ctrl, ksba_reader_t reader,
   tmpfp = tmpfile ();
   if (!tmpfp)
     {
-      err = gpg_error_from_errno (errno);
+      err = gpg_error_from_syserror ();
       log_error (_("error creating temporary file: %s\n"), strerror (errno));
       goto cleanup;
     }
@@ -528,7 +529,7 @@ parse_p12 (ctrl_t ctrl, ksba_reader_t reader,
     {
       if (nread && fwrite (buffer, nread, 1, tmpfp) != 1)
         {
-          err = gpg_error_from_errno (errno);
+          err = gpg_error_from_syserror ();
           log_error (_("error writing to temporary file: %s\n"),
                      strerror (errno));
           goto cleanup;
@@ -545,7 +546,7 @@ parse_p12 (ctrl_t ctrl, ksba_reader_t reader,
   certfp = tmpfile ();
   if (!certfp)
     {
-      err = gpg_error_from_errno (errno);
+      err = gpg_error_from_syserror ();
       log_error (_("error creating temporary file: %s\n"), strerror (errno));
       goto cleanup;
     }
